﻿using System;
using System.Linq;
using System.Runtime.InteropServices;

class Program
{
    [DllImport("Dll1.dll")]
    public unsafe static extern void FixedStructTest(ref TestStruct test, int bufLen);

    static unsafe void Main(string[] args)
    {
        TestStruct ts = new TestStruct();

        ts.shortField = 5;
        ts.intField = 100;
        ts.Text = "test is good";

        for (int i = 0; i < TestStruct.MaxLength; i++)
        {
            ts.fields[i] = (i * 2) + 100;
        }

        Console.WriteLine("[C# output (passing to C++)]");
        OutputStruct(ts);
        Console.WriteLine();
        Console.WriteLine();

        FixedStructTest(ref ts, TestStruct.MaxLength);

        Console.WriteLine("[C# output (from C++)]");
        OutputStruct(ts);
        Console.WriteLine();
    }

    static unsafe void OutputStruct(in TestStruct ts)
    {
        Console.WriteLine($"{ts.shortField}, {ts.intField}, {ts.Text}");

        for (int i = 0; i < TestStruct.MaxLength; i++)
        {
            Console.Write($"{ts.fields[i]},");
        }
    }
}

[StructLayout(LayoutKind.Sequential, Pack = 1)]
public unsafe struct TestStruct
{
    public const int MaxLength = 20;

    public short shortField;
    public int intField;
    fixed char txt[MaxLength];
    public fixed long fields[MaxLength];

    public unsafe string Text
    {
        get
        {
            fixed (char* pText = txt) return new string(pText);
        }

        set
        {
            if (value.Length > MaxLength)
            {
                throw new ArgumentOutOfRangeException();
            }

            fixed (char* pDst = txt) Marshal.Copy(value.ToArray(), 0, new IntPtr(pDst), value.Length);
        }
    }
}
